#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from bs4 import BeautifulSoup

def _findRawContent(item):
	if item.has_attr('content'):
		title = item['content'].strip()
		if title:
			return title
	return item.text.strip()

def fact():
	return BeautifulSoup("<div></div>", features="html.parser")

def _copyB(soup):
	return BeautifulSoup(str(soup), features="html.parser")

def _seemsValidText(soup, limit=500):
	if not soup:
		return False
	return soup.text and len(soup.text) > limit

def _wrap(name, *children):
	r = fact().new_tag(name)
	for child in children:
		r.append(child)
	return r


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import requests
from bs4 import BeautifulSoup
from readability import Document
from .content import _findMain
import hashlib
import sys
import os

TO_REMOVE = [
	'跳到导航', 
	'跳到搜索', 
	'Skip to main content'
]

def _findUrl(url, soup):
	if 'telegra.ph' not in url:
		return
	address = soup.find('address')
	if not address:
		return
	link = address.find('a')
	return link and link.get('href')

def _trimWebpage(raw):
	for to_remove in TO_REMOVE:
		raw = raw.replace(to_remove, '')
	anchor = '<!-- detail_toolbox -->'
	index = raw.find(anchor)
	if index != -1:
		return raw[:index]
	return raw

def _getUrlContent(url):
	headers = {
		'method': 'GET',
		'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3',
		'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/78.0.3904.97 Safari/537.36',
	}
	return requests.get(url, headers=headers).text

def _cachedContent(url):
	os.system('mkdir tmp > /dev/null 2>&1')
	cache = 'tmp/' + hashlib.sha224(url.encode('utf-8')).hexdigest()[:10] + '.html'
	try:
		with open(cache) as f:
			return f.read()
	except:
		content = _getUrlContent(url)
		with open(cache, 'w') as f:
			f.write(content)
		return content

def getArticle(url, content, args = {}):
	if not content:
		if 'test' in str(sys.argv):
			content = _cachedContent(url)
		else:
			content = _getUrlContent(url)
	soup = BeautifulSoup(_trimWebpage(content), 'html.parser')
	article_url = _findUrl(url, soup) # may need to use
	doc = Document(content)
	return _findMain(soup, doc, url, args)




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from .common import _seemsValidText
from telegram_util import matchKey

SHORT_ARTICLE = ['每日书摘']

def _seemsValidRawArticle(soup, text_limit = 500):
	if not _seemsValidText(soup, limit = text_limit):
		return False
	return not not soup.find('img')

def _getInnerArticle_(soup):	
	applicators = [
		lambda x: x.find("article"),
		lambda x: x.find("div", {"id" : "js_content"}),
		lambda x: x.find("div", class_ = "story-body__inner"),
		lambda x: x.find("div", class_ = "answercell"),
		lambda x: x.find("div", class_ = "post-text"),
		lambda x: x.find("div", {"id" : "bodyContent"}),
		lambda x: x.find("div", {"id" : "content_JS"}),
		lambda x: x.find("div", {"id" : "link-report"}),
		lambda x: x.find("div", class_ = "main-post"),
		lambda x: x.find("div", class_ = "article"),
	]
	is_short = matchKey(soup.text, SHORT_ARTICLE)
	text_limit = 150 if is_short else 500
	for applicator in applicators:
		candidate = applicator(soup)
		if _seemsValidRawArticle(candidate, text_limit = text_limit):
			soup = candidate
	return soup

def _getInnerArticle(soup):
	all_content = str(soup)
	inner = _getInnerArticle_(soup)
	index = all_content.find(str(inner))
	if index == -1:
		return inner, None
	return inner, all_content[:index]


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from .common import _findRawContent

def _parseDomain(url):
	if not url:
		return 
	if not url.startswith('http'):
		return
	r = '/'.join(url.split('/')[:3])
	if r.count('/') == 2 and 'http' in r:
		return r

def _findDomain(soup, url):
	for meta in soup.find_all('meta'):
		for att in meta.attrs:
			if 'url' in att.lower():
				r = _parseDomain(meta[att])
				if r:
					return r
	for meta in soup.find_all('meta'):
		if 'url' in str(meta).lower():
			r = _parseDomain(_findRawContent(meta))
			if r:
				return r
	return _parseDomain(url)